/* Copyright (c) 2004-2011 dhpoware. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without 
 * restriction, including without limitation the rights to use, 
 * copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom 
 * the Software is furnished to do so, subject to the following 
 * conditions:
 *
 * The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <assert.h>
#include "win32_dib.h"

void DIBblt(DIB *pDIB, HDC hdcDest, int x, int y)
{
    assert(pDIB != NULL && hdcDest != NULL);
    StretchBlt(hdcDest, x, y, pDIB->width, pDIB->height, pDIB->dc,
        0, 0, pDIB->width, pDIB->height, SRCCOPY);
}

void DIBbltStrech(DIB *pDIB, HDC hdcDest, int x, int y,
                  int destWidth, int destHeight)
{
    assert(pDIB != NULL && hdcDest != NULL);
    StretchBlt(hdcDest, x, y, destWidth, destHeight, pDIB->dc,
        0, 0, pDIB->width, pDIB->height, SRCCOPY);
}

void DIBcopyByteAligned(DIB *pDIB, BYTE *pTarget)
{
    int y, targetPitch;

    if (!pDIB || !pTarget)
        return;
    
    targetPitch = pDIB->width * 3;

    for (y = 0; y < pDIB->height; ++y)
    {
        memcpy(&pTarget[targetPitch * y],
            &pDIB->pPixels[pDIB->pitch * y],
            targetPitch);
    }
}

BOOL DIBcreate(DIB *pDIB, int width, int height)
{
    if (!pDIB)
        return FALSE;

    memset(pDIB, 0, sizeof(DIB));

    pDIB->width = width;
    pDIB->height = height;
    pDIB->pitch = ((width * 24 + 31) & ~31) >> 3;
    pDIB->dc = CreateCompatibleDC(NULL);

    if (!pDIB->dc)
        return FALSE;

    pDIB->info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    pDIB->info.bmiHeader.biBitCount = 24;
    pDIB->info.bmiHeader.biWidth = width;
    pDIB->info.bmiHeader.biHeight = -height;
    pDIB->info.bmiHeader.biCompression = BI_RGB;
    pDIB->info.bmiHeader.biPlanes = 1;

    pDIB->hBitmap = CreateDIBSection(pDIB->dc, &pDIB->info,
        DIB_RGB_COLORS, (void**)&pDIB->pPixels, NULL, 0);

    if (!pDIB->hBitmap)
    {
        DIBdestroy(pDIB);
        return FALSE;
    }

    GdiFlush();
    return TRUE;
}

void DIBdeselect(DIB *pDIB)
{
    if (!pDIB)
        return;

    if (!pDIB->dc)
        return;
    
    if (!pDIB->hPrevObj)
        return;

    SelectObject(pDIB->dc, pDIB->hPrevObj);
    pDIB->hPrevObj = NULL;
}

void DIBdestroy(DIB *pDIB)
{
    if (!pDIB)
        return;

    DIBdeselect(pDIB);

    if (pDIB->hBitmap)
    {
        DeleteObject(pDIB->hBitmap);
        pDIB->hBitmap = NULL;
    }

    if (pDIB->dc)
    {
        DeleteDC(pDIB->dc);
        pDIB->dc = NULL;
    }
}

void DIBselect(DIB *pDIB)
{
    if (!pDIB)
        return;

    if (!pDIB->dc)
        return;

    pDIB->hPrevObj = SelectObject(pDIB->dc, pDIB->hBitmap);
}